#include "grid.h"

namespace Msp {
namespace GLtk {

Grid::Grid(Layout &l, unsigned c):
	Arrangement(l),
	columns(c),
	first_row(true),
	column(0)
{ }

void Grid::skip()
{
	finish_slot();
}

void Grid::next_row()
{
	if(row_bottom.empty())
		throw arrangement_error("empty row not allowed");

	finish_row();
}

void Grid::process_widget(Widget &wdg, Side side, bool aligned)
{
	if(side==TOP)
	{
		Edge &top_edge = (first_row ? edges[TOP] : row_top);
		if(top_edge.aligned && aligned)
			add_constraint(wdg, Layout::ALIGN_TOP, top_edge);

		bool snug = (edges[BOTTOM].aligned && aligned);
		add_constraint(wdg, (snug ? Layout::BELOW : Layout::FAR_BELOW), BOTTOM);

		top_edge.add(wdg, aligned);
	}
	else if(side==BOTTOM)
	{
		if(row_bottom.aligned && aligned)
			add_constraint(wdg, Layout::ALIGN_BOTTOM, row_bottom);

		row_bottom.add(wdg, aligned);
	}
	else if(side==LEFT)
	{
		if(columns[column].left.aligned && aligned)
			add_constraint(wdg, Layout::ALIGN_LEFT, columns[column].left);
		else if(column>0)
		{
			bool snug = (columns[column-1].right.aligned && aligned);
			add_constraint(wdg, (snug ? Layout::RIGHT_OF : Layout::FAR_RIGHT_OF), columns[column-1].right);
		}

		edges[LEFT].add(wdg, (aligned && !column));
		columns[column].left.add(wdg, aligned);
	}
	else if(side==RIGHT)
	{
		if(columns[column].right.aligned && aligned)
			add_constraint(wdg, Layout::ALIGN_RIGHT, columns[column].right);

		edges[RIGHT].add(wdg, (aligned && column+1==columns.size()));
		columns[column].right.add(wdg, aligned);
	}
}

void Grid::finish_widget(Widget &wdg)
{
	layout.set_gravity(wdg, -1, 1);
}

void Grid::finish_slot()
{
	++column;
	if(column==columns.size())
		finish_row();
}

void Grid::finish_row()
{
	edges[BOTTOM] = row_bottom;
	row_bottom.clear();
	row_top.clear();
	first_row = false;
	column = 0;
}


Grid::Loader::Loader(Grid &g):
	DataFile::ObjectLoader<Grid>(g)
{
	add("next_row", &Loader::next_row);
	add("skip",     &Loader::skip);
}

void Grid::Loader::next_row()
{
	obj.next_row();
}

void Grid::Loader::skip()
{
	obj.skip();
}

} // namespace GLtk
} // namespace Msp
