#include "panel.h"
#include "part.h"
#include "style.h"
#include "toggle.h"

using namespace std;

namespace Msp {
namespace GLtk {

Toggle::Toggle(const string &t):
	text(),
	pressed(false),
	value(false),
	exclusive(false)
{
	set_text(t);
}

void Toggle::autosize_special(const Part &part, Geometry &ageom)
{
	if(part.get_name()=="text")
		text.autosize(part, ageom);
}

void Toggle::set_text(const string &t)
{
	text = t;
	signal_autosize_changed.emit();
	rebuild();
}

void Toggle::set_exclusive(bool e)
{
	exclusive = e;
	if(exclusive && value)
		exclude_siblings();
}

void Toggle::exclude_siblings()
{
	const list<Widget *> &siblings = parent->get_children();
	for(list<Widget *>::const_iterator i=siblings.begin(); i!=siblings.end(); ++i)
		if(Toggle *tgl = dynamic_cast<Toggle *>(*i))
			if(tgl!=this && tgl->get_exclusive() && tgl->get_value())
				tgl->set_value(false);
}

void Toggle::set_value(bool v)
{
	bool old_value = value;

	value = v;
	if(value)
	{
		set_state(ACTIVE);
		if(exclusive && parent)
			exclude_siblings();
	}
	else
		clear_state(ACTIVE);

	if(value!=old_value)
		signal_toggled.emit(value);
}

void Toggle::rebuild_special(const Part &part)
{
	if(part.get_name()=="text")
		text.build(part, state, geom, part_cache);
}

void Toggle::button_press(int, int, unsigned btn)
{
	if(btn==1)
		pressed = true;
}

void Toggle::button_release(int x, int y, unsigned btn)
{
	if(pressed && btn==1)
	{
		if(geom.is_inside_relative(x, y) && (!value || !exclusive))
			set_value(!value);

		pressed = false;
	}
}

void Toggle::on_style_change()
{
	text.set_style(style);
}


Toggle::Loader::Loader(Toggle &t):
	DataFile::DerivedObjectLoader<Toggle, Widget::Loader>(t)
{
	add("exclusive", &Toggle::exclusive);
	add("text",      &Loader::text);
	add("value",     &Toggle::value);
}

void Toggle::Loader::finish()
{
	obj.set_state(ACTIVE, (obj.value ? ACTIVE : NORMAL));
}

void Toggle::Loader::text(const string &t)
{
	obj.text = t;
}

} // namespace GLtk
} // namespace Msp
